#!/usr/bin/env python3
# -*- coding: utf-8 -*

import click
import requests
import json
import os
import tarfile
import shutil
import time
import datetime
import sys

script_version = '0.0.17'
date = datetime.datetime.today()

ROOT_DIRECTORY = os.path.join(os.path.expanduser('~'), '.abeja')
config_file_path = os.path.join(ROOT_DIRECTORY, 'config')

CONFIG = None
ABEJA_PLATFORM_USER_ID = None
ABEJA_PLATFORM_TOKEN = None
ORGANIZATION_NAME = None
ORGANIZATION_ENDPOINT = None
if os.path.exists(config_file_path):
    with open(config_file_path, 'r') as f:
        CONFIG = json.load(f)
        ABEJA_PLATFORM_USER_ID = CONFIG['abeja-platform-user']
        ABEJA_PLATFORM_TOKEN = CONFIG['personal-access-token']
        ORGANIZATION_NAME = CONFIG['organization-name']
        ORGANIZATION_ENDPOINT = "https://api.dev.abeja.io/organizations/{}".format(ORGANIZATION_NAME)


@click.group(help='abeja cli tools')
@click.version_option(version=script_version, message='v%(version)s')
@click.pass_context
def main(ctx):
    pass


@main.group(help='model operation commands')
@click.pass_context
def model(ctx):
    pass


# ---------------------------------------------------
# Configure
# ---------------------------------------------------
@main.command(help='Set ABEJA credentials')
@click.pass_context
def configure(ctx):
    user = input('abeja-platform-user  : ')
    token = input('personal-access-token: ')
    org = input('organization-name    : ')

    config = {
        "abeja-platform-user": "user-{}".format(user),
        "personal-access-token": token,
        "organization-name": org
    }

    _create_configfile(config)
    click.echo('[INFO]: ABEJA credentials setup completed!')


def _create_configfile(config):
    if not os.path.exists(ROOT_DIRECTORY):
        os.makedirs(ROOT_DIRECTORY, mode=0o711)

    with open(config_file_path, "w") as f:
        json.dump(config, f)


# ---------------------------------------------------
# models
# ---------------------------------------------------
@model.command(name='create-model', help='create model')
@click.option('-n', '--name', 'name', type=str, help='model name', required=True)
@click.option('-d', '--description', 'description', type=str, help='model description', required=True)
@click.option('-i', '--init_project', 'init_project', is_flag=True, help='set if want to init project')
def create_model(name, description, init_project):
    r = _create_model(name, description, init_project)

    click.echo(r)


def _init_project(name):
    if not os.path.exists(name):
        os.makedirs(name)
    _download(name, "https://s3-us-west-2.amazonaws.com/abeja-platform-config-dev/cli/main.py")


def _create_model(name, description, init_project=False):
    parameter = {
        'name': name,
        'description': description,
    }
    json_data = json.dumps(parameter)

    # create directory & sample code download
    if init_project:
        _init_project(name)

    url = "{}/models".format(ORGANIZATION_ENDPOINT)
    r = _post(url, json_data)

    return r


@model.command(name='describe-models', help='get to model or endpoint model')
@click.option('-m', '--model_id', 'model_id', type=str, help='model identifier', default='all', required=False)
def describe_models(model_id):
    r = _describe_models(model_id)

    click.echo(r)


def _describe_models(model_id):
    url = '{}/models'.format(ORGANIZATION_ENDPOINT)
    if model_id != "all":
        url = '{}/{}'.format(url, model_id)
    r = _get(url)

    return r


@model.command(name='delete-model', help='delete model')
@click.option('-m', '--model_id', 'model_id', type=str, help='model identifier', required=True)
def delete_model(model_id):
    r = _delete_model(model_id)

    click.echo(r)


def _delete_model(model_id):
    url = '{}/models/{}'.format(ORGANIZATION_ENDPOINT, model_id)
    r = _delete(url)

    return r


# ---------------------------------------------------
# models versions
# ---------------------------------------------------
@model.command(name='create-version', help='create version & upload application')
@click.pass_context
@click.option('-m', '--model_id', 'model_id', type=str, help='model identifier', required=True)
@click.option('-v', '--version', 'version', type=str, help='model version code', required=True)
@click.option('-i', '--image', 'image', type=str, help='base-image name', required=True)
def create_version(ctx, model_id, version, image):
    r = _create_version(ctx, model_id, version, image)
    click.echo(r)


def _create_version(ctx, model_id, version, image):
    parameter = {
        'image': image,
        'version': version,
    }
    json_data = json.dumps(parameter)

    url = '{}/models/{}/versions'.format(ORGANIZATION_ENDPOINT, model_id)
    r = _post(url, json_data)
    r = _json_loads(r)

    upload_url = r['upload_url']
    name = _get_model_name(model_id)
    tar_name = _version_archive(name)

    _version_upload(upload_url, tar_name)

    return r


def _get_model_name(model_id):
    url = '{}/models/{}'.format(ORGANIZATION_ENDPOINT, model_id)
    r = _get(url)
    r = _json_loads(r)

    return r['name']


def _version_archive(dirname):
    execlude_filenames = ['.git']
    tar_name = dirname + '.tar.gz'
    filenames = os.listdir(dirname)
    with tarfile.open(tar_name, mode='w:gz') as archive:
        for filename in filenames:
            if filename in execlude_filenames:
                continue
            filepath = os.path.join(dirname, filename)
            archive.add(filepath, arcname=filename)
    return tar_name


def _version_upload(upload_url, tar_name):
    with open(tar_name, "rb") as file:
        result = requests.put(upload_url, data=file)

    return result


@model.command(name='describe-versions', help='get to version or version list')
@click.option('-m', '--model_id', 'model_id', type=str, help='model identifier', required=True)
@click.option('-v', '--version_id', 'version_id', type=str, help='version identifier', default='all', required=False)
def describe_versions(model_id, version_id):
    r = _describe_versions(model_id, version_id)

    click.echo(r)


def _describe_versions(model_id, version_id):
    url = '{}/models/{}/versions'.format(ORGANIZATION_ENDPOINT, model_id)
    if version_id != "all":
        url = '{}/{}'.format(url, version_id)
    r = _get(url)

    return r


@model.command(name='delete-version', help='delete version')
@click.option('-m', '--model_id', 'model_id', type=str, help='model identifier', required=True)
@click.option('-v', '--version_id', 'version_id', type=str, help='version identifier', required=True)
def delete_version(model_id, version_id):
    r = _delete_version(model_id, version_id)

    click.echo(r)


def _delete_version(model_id, version_id):
    url = '{}/models/{}/versions/{}'.format(ORGANIZATION_ENDPOINT, model_id, version_id)
    r = _delete(url)

    return r


# ---------------------------------------------------
# deployments
# ---------------------------------------------------
@model.command(name='create-deployment', help='deploy a specific model')
@click.option('-n', '--name', 'name', type=str, help='deployment name', required=True)
@click.option('-m', '--model_id', 'model_id', type=str, help='model identifier', required=True)
def create_deployment(name, model_id):
    r = _create_deployment(name, model_id)
    click.echo(r)


def _create_deployment(name, model_id):
    parameter = {'name': name}
    json_data = json.dumps(parameter)

    url = "{}/models/{}/deployments".format(ORGANIZATION_ENDPOINT, model_id)
    r = _post(url, json_data)

    return r


@model.command(name='describe-deployments', help='get to deployment or deployment list')
@click.option('-d', '--deployment_id', 'deployment_id', type=str, help='deployment identifier', default='all',
              required=False)
def describe_deployments(deployment_id):
    r = _describe_deployments(deployment_id)

    click.echo(r)


def _describe_deployments(deployment_id):
    url = '{}/deployments'.format(ORGANIZATION_ENDPOINT)
    if deployment_id != "all":
        url = '{}/{}'.format(url, deployment_id)
    r = _get(url)

    return r


@model.command(name='delete-deployment', help='delete deployment')
@click.option('-d', '--deployment_id', 'deployment_id', type=str, help='deployment identifier', required=True)
def delete_deployment(deployment_id):
    r = _delete_deployment(deployment_id)

    click.echo(r)


def _delete_deployment(deployment_id):
    url = '{}/deployments/{}'.format(ORGANIZATION_ENDPOINT, deployment_id)
    r = _delete(url)

    return r


# ---------------------------------------------------
# service
# ---------------------------------------------------
@model.command(name='create-service', help='create http service')
@click.option('-d', '--deployment_id', 'deployment_id', type=str, help='deployment identifier', required=True)
@click.option('-v', '--version_id', 'version_id', type=str, help='version identifier', required=True)
@click.option('-a', '--alias', 'alias', type=str, help='http endpoint alias name', required=True)
def create_service(deployment_id, version_id, alias):
    r = _create_service(deployment_id, version_id, alias)
    click.echo(r)


def _create_service(deployment_id, version_id, alias):
    parameter = {
        'version_id': version_id,
        'alias': alias,
    }
    json_data = json.dumps(parameter)

    url = "{}/deployments/{}/services".format(ORGANIZATION_ENDPOINT, deployment_id)
    r = _post(url, json_data)

    return r


@model.command(name='describe-services', help='get to service or service list')
@click.option('-d', '--deployment_id', 'deployment_id', type=str, help='deployment identifier', required=True)
@click.option('-s', '--service_id', 'service_id', type=str, help='service identifier', default='all', required=False)
def describe_services(deployment_id, service_id):
    r = _describe_services(deployment_id, service_id)

    click.echo(r)


def _describe_services(deployment_id, service_id):
    url = '{}/deployments/{}/services'.format(ORGANIZATION_ENDPOINT, deployment_id)
    if service_id != "all":
        url = '{}/{}'.format(url, service_id)
    r = _get(url)

    return r


@model.command(name='delete-service', help='delete service')
@click.option('-d', '--deployment_id', 'deployment_id', type=str, help='deployment identifier', required=True)
@click.option('-s', '--service_id', 'service_id', type=str, help='service identifier', required=True)
def delete_service(deployment_id, service_id):
    r = _delete_service(deployment_id, service_id)

    click.echo(r)


def _delete_service(deployment_id, service_id):
    url = '{}/deployments/{}/services/{}'.format(ORGANIZATION_ENDPOINT, deployment_id, service_id)
    r = _delete(url)

    return r


# ---------------------------------------------------
# endpoint
# ---------------------------------------------------
@model.command(name='create-endpoint', help='create endpoint')
@click.option('-d', '--deployment_id', 'deployment_id', type=str, help='deployment identifier', required=True)
@click.option('-s', '--service_id', 'service_id', type=str, help='service identifier', required=True)
@click.option('-c', '--custom_alias', 'custom_alias', type=str, help='custom_alias name', required=True)
def create_endpoint(deployment_id, service_id, custom_alias):
    r = _create_endpoint(deployment_id, service_id, custom_alias)
    click.echo(r)


def _create_endpoint(deployment_id, service_id, custom_alias):
    parameter = {
        'service_id': service_id,
        'custom_alias': custom_alias,
    }
    json_data = json.dumps(parameter)

    url = "{}/deployments/{}/endpoints".format(ORGANIZATION_ENDPOINT, deployment_id)
    r = _post(url, json_data)

    return r


@model.command(name='describe-endpoints', help='get to endpoint or endpoint list')
@click.option('-d', '--deployment_id', 'deployment_id', type=str, help='deployment identifier', required=True)
@click.option('-e', '--endpoint_id', 'endpoint_id', type=str, help='endpoint identifier', default='all', required=False)
def describe_endpoints(deployment_id, endpoint_id):
    r = _describe_endpoints(deployment_id, endpoint_id)

    click.echo(r)


def _describe_endpoints(deployment_id, endpoint_id):
    url = '{}/deployments/{}/endpoints'.format(ORGANIZATION_ENDPOINT, deployment_id)
    if endpoint_id != "all":
        url = '{}/{}'.format(url, endpoint_id)
    r = _get(url)

    return r


@model.command(name='delete-endpoint', help='delete endpoint')
@click.option('-d', '--deployment_id', 'deployment_id', type=str, help='deployment identifier', required=True)
@click.option('-e', '--endpoint_id', 'endpoint_id', type=str, help='endpoint identifier', required=True)
def delete_endpoint(deployment_id, endpoint_id):
    r = _delete_endpoint(deployment_id, endpoint_id)

    click.echo(r)


def _delete_endpoint(deployment_id, endpoint_id):
    url = '{}/deployments/{}/endpoints/{}'.format(ORGANIZATION_ENDPOINT, deployment_id, endpoint_id)
    r = _delete(url)

    return r


@model.command(name='update-endpoint', help='update endpoint')
@click.option('-d', '--deployment_id', 'deployment_id', type=str, help='deployment identifier', required=True)
@click.option('-s', '--service_id', 'service_id', type=str, help='service identifier', required=True)
@click.option('-e', '--endpoint_id', 'endpoint_id', type=str, help='endpoint identifier', required=True)
def update_endpoint(deployment_id, service_id, endpoint_id):
    r = _update_endpoint(deployment_id, service_id, endpoint_id)
    click.echo(r)


def _update_endpoint(deployment_id, service_id, endpoint_id):
    parameter = {
        'service_id': service_id,
    }
    jsonData = json.dumps(parameter)

    url = "{}/deployments/{}/endpoints/{}".format(ORGANIZATION_ENDPOINT, deployment_id, endpoint_id)
    r = _patch(url, jsonData)

    return r


@model.command(name='check-endpoint-json', help='check endpoint json')
@click.option('-d', '--deployment_id', 'deployment_id', type=str, help='deployment identifier', required=True)
@click.option('-k', '--key', 'key', type=str, help='key', required=True)
@click.option('-v', '--value', 'value', type=str, help='value', required=True)
def check_endpoint_json(deployment_id, key, value):
    r = _check_endpoint_json(deployment_id, key, value)
    click.echo(r)


def _check_endpoint_json(deployment_id, key, value):
    config = _json_file_load(config_file_path)
    parameter = {
        key: value,
    }
    jsonData = json.dumps(parameter)

    url = "https://{}.api.dev.abeja.io/deployments/{}".format(config['organization-name'], deployment_id)
    r = requests.post(url, jsonData, headers={'Content-Type': 'application/json'},
                      auth=(config['abeja-platform-user'], config['personal-access-token']))

    return r


@model.command(name='check-endpoint-image', help='check endpoint image')
@click.option('-d', '--deployment_id', 'deployment_id', type=str, help='deployment identifier', required=True)
@click.option('-t', '--type', 'type', type=str, help='contant type', required=True)
@click.option('-i', '--image', 'image', type=str, help='image file', required=True)
def check_endpoint_image(deployment_id, type, image):
    r = _check_endpoint_image(deployment_id, type, image)
    click.echo(r)


def _check_endpoint_image(deployment_id, type, image):
    config = _json_file_load(config_file_path)
    url = "https://{}.api.dev.abeja.io/deployments/{}".format(config['organization-name'], deployment_id)
    contant_type = "image/{}".format(type)
    data = open(image, "rb").read()

    r = requests.post(url, data=data, headers={'Content-Type': contant_type},
                      auth=(config['abeja-platform-user'], config['personal-access-token']))

    return r


# ---------------------------------------------------
# submit-run
# ---------------------------------------------------
@model.command(name='submit-run', help='submit run')
@click.option('-d', '--deployment_id', 'deployment_id', type=str, help='deployment identifier', required=True)
@click.option('-v', '--version_id', 'version_id', type=str, help='version identifier', required=True)
@click.option('-i', '--input_operator', 'input_operator', type=str, help='input oparator', required=True)
@click.option('-i', '--input_target', 'input_target', type=str, help='input target', required=True)
@click.option('-o', '--output_operator', 'output_operator', type=str, help='output operator', required=True)
@click.option('-i', '--output_target', 'output_target', type=str, help='output target', required=True)
def submit_run(deployment_id, version_id, input_operator, input_target, output_operator, output_target):
    r = _submit_run(deployment_id, version_id, input_operator, input_target, output_operator, output_target)

    click.echo(r)


def _submit_run(deployment_id, version_id, input_operator, input_target, output_operator, output_target):
    parameter = {
        'version_id': version_id,
        'input_data': {input_operator: input_target},
        'output_template': {output_operator: output_target},
    }
    json_data = json.dumps(parameter)
    url = "{}/deployments/{}/runs".format(ORGANIZATION_ENDPOINT, deployment_id)
    r = _post(url, json_data)

    return r


@model.command(name='describe-runs', help='get to run or run list')
@click.option('-d', '--deployment_id', 'deployment_id', type=str, help='deployment identifier', required=True)
@click.option('-r', '--run_id', 'run_id', type=str, help='run identifier', default='all', required=False)
def describe_runs(deployment_id, run_id):
    r = _describe_runs(deployment_id, run_id)

    click.echo(r)


def _describe_runs(deployment_id, run_id):
    url = '{}/deployments/{}/runs'.format(ORGANIZATION_ENDPOINT, deployment_id)
    if run_id != "all":
        url = '{}/{}'.format(url, run_id)
    r = _get(url)

    return r


@model.command(name='delete-run', help='delete run')
@click.option('-d', '--deployment_id', 'deployment_id', type=str, help='deployment identifier', required=True)
@click.option('-r', '--run_id', 'run_id', type=str, help='run identifier', required=True)
def delete_run(deployment_id, run_id):
    r = delete_run(deployment_id, run_id)

    click.echo(r)


def _delete_run(deployment_id, run_id):
    url = '{}/deployments/{}/runs/{}'.format(ORGANIZATION_ENDPOINT, deployment_id, run_id)
    r = _delete(url)

    return r


# ---------------------------------------------------
# trigger
# ---------------------------------------------------
@model.command(name='create-trigger', help='create trigger')
@click.option('-d', '--deployment_id', 'deployment_id', type=str, help='deployment identifier', required=True)
@click.option('-v', '--version_id', 'version_id', type=str, help='version identifier', required=True)
@click.option('--input_service_name', 'input_service_name', type=str, help='input service name', required=True)
@click.option('--input_service_id', 'input_service_id', type=str, help='input service id', required=True)
@click.option('--output_service_name', 'output_service_name', type=str, help='output service name', required=True)
@click.option('--output_service_id', 'output_service_id', type=str, help='output service id', required=True)
def create_trigger(deployment_id, version_id, input_service_name, input_service_id, output_service_name,
                   output_service_id):
    r = _create_trigger(deployment_id, version_id, input_service_name, input_service_id, output_service_name,
                        output_service_id)
    click.echo(r)


def _create_trigger(deployment_id, version_id, input_service_name, input_service_id, output_service_name,
                    output_service_id):
    parameter = {
        'version_id': version_id,
        'input_service_name': input_service_name,
        'input_service_id': input_service_id,
        'output_service_name': output_service_name,
        'output_service_id': output_service_id,
    }
    json_data = json.dumps(parameter)

    url = "{}/deployments/{}/triggers".format(ORGANIZATION_ENDPOINT, deployment_id)
    r = _post(url, json_data)

    return r


@model.command(name='describe-triggers', help='get to trigger or trigger list')
@click.option('-d', '--deployment_id', 'deployment_id', type=str, help='deployment identifier', required=True)
@click.option('-t', '--trigger_id', 'trigger_id', type=str, help='trigger identifier', default='all', required=False)
def describe_triggers(deployment_id, trigger_id):
    r = _describe_triggers(deployment_id, trigger_id)

    click.echo(r)


def _describe_triggers(deployment_id, trigger_id):
    url = '{}/deployments/{}/triggers'.format(ORGANIZATION_ENDPOINT, deployment_id)
    if trigger_id != "all":
        url = '{}/{}'.format(url, trigger_id)
    r = _get(url)

    return r


@model.command(name='delete-trigger', help='delete trigger')
@click.option('-d', '--deployment_id', 'deployment_id', type=str, help='deployment identifier', required=True)
@click.option('-t', '--trigger_id', 'trigger_id', type=str, help='trigger identifier', required=True)
def delete_trigger(deployment_id, trigger_id):
    r = _delete_trigger(deployment_id, trigger_id)

    click.echo(r)


def _delete_trigger(deployment_id, trigger_id):
    url = '{}/deployments/{}/triggers/{}'.format(ORGANIZATION_ENDPOINT, deployment_id, trigger_id)
    r = _delete(url)

    return r


# ---------------------------------------------------
# log
# ---------------------------------------------------
@model.command(name='describe-service-logs', help='get service log')
@click.option('-d', '--deployment_id', 'deployment_id', type=str, help='deployment identifier', required=True)
@click.option('-s', '--service_id', 'service_id', type=str, help='service identifier', required=True)
@click.option('--start_time', 'start_time', type=str, help='start time / 2017-01-01T00:00:00Z', default='none',
              required=False)
@click.option('--end_time', 'end_time', type=str, help='end time / 2017-12-31T00:00:00Z', default='none',
              required=False)
@click.option('--next_token', 'next_token', type=str, help='next token', default='none', required=False)
def describe_service_logs(deployment_id, service_id, start_time, end_time, next_token):
    parameter = {
        'start_time': start_time,
        'end_time': end_time,
        'next_token': next_token,
    }
    json_data = json.dumps(parameter)

    url = "{}/deployments/{}/services/{}/logs".format(ORGANIZATION_ENDPOINT, deployment_id, service_id)
    r = _get_data(url, json_data)

    return click.echo(r)


@model.command(name='describe-run-logs', help='get run log')
@click.option('-d', '--deployment_id', 'deployment_id', type=str, help='deployment identifier', required=True)
@click.option('-r', '--run_id', 'run_id', type=str, help='run identifier', required=True)
@click.option('--start_time', 'start_time', type=str, help='start time', default='none', required=False)
@click.option('--end_time', 'end_time', type=str, help='end time', default='none', required=False)
@click.option('--next_token', 'next_token', type=str, help='next token', default='none', required=False)
def describe_run_logs(deployment_id, run_id, start_time, end_time, next_token):
    parameter = {
        'start_time': start_time,
        'end_time': end_time,
        'next_token': next_token,
    }
    json_data = json.dumps(parameter)

    url = "{}/deployments/{}/runs/{}/logs".format(ORGANIZATION_ENDPOINT, deployment_id, run_id)
    r = _get_data(url, json_data)

    return click.echo(r)


# ---------------------------------------------------
# simple command
# ---------------------------------------------------
@model.command(name='simple-deploy', help='simple deploy command')
@click.option('-n', '--name', 'name', type=str, help='model name', default='default', required=False)
@click.option('-i', '--init_project', 'init_project', is_flag=True, default=True, help='set if want to init project')
@click.option('-d', '--description', 'description', type=str, help='model description', default='default description',
              required=False)
@click.option('-v', '--version', 'version', type=str, help='model version code', default='0.0.1', required=False)
@click.option('-i', '--image', 'image', type=str, help='base-image name', default='abeja-inc/minimal:0.1.0',
              required=False)
@click.option('-a', '--alias', 'alias', type=str, help='http endpoint alias name', default='prod', required=False)
@click.option('-c', '--custom_alias', 'custom_alias', type=str, help='custom alias name',
              default='default', required=False)
@click.pass_context
def simple_deploy(ctx, name, description, version, image, alias, custom_alias, init_project):
    model = _create_model(name, description, init_project)
    model = _json_loads(model)
    click.echo("[info] create model. // model_id: " + model['model_id'])

    version = _create_version(ctx, model['model_id'], version, image)
    click.echo("[info] create version and model upload. // version_id: " + version['version_id'])

    deployment = _create_deployment(name, model['model_id'])
    deployment = _json_loads(deployment)
    click.echo("[info] create deployment. // deployment_id: " + deployment['deployment_id'])

    service = _create_service(deployment['deployment_id'], version['version_id'], alias)
    service = _json_loads(service)
    click.echo("[info] create service. // service_id: " + service['service_id'])

    time.sleep(10.0)
    endpoint = _create_endpoint(deployment['deployment_id'], service['service_id'], custom_alias)
    endpoint = _json_loads(endpoint)
    click.echo("[info] create endpoint. // endpoint_id: " + endpoint['endpoint_id'])

    url = 'https://{}.api.dev.abeja.io/deployments/{}'.format(CONFIG['organization-name'], deployment['deployment_id'])
    click.echo("[info] url: " + url)


@model.command(name='simple-update', help='simple update command')
@click.option('-m', '--model_id', 'model_id', type=str, help='model identifier', required=True)
@click.option('-v', '--version', 'version', type=str, help='model version code', default='0.0.1', required=True)
@click.option('-i', '--image', 'image', type=str, help='base-image name', default='abeja-inc/minimal:0.1.0',
              required=True)
@click.option('-d', '--deployment_id', 'deployment_id', type=str, help='deployment identifier', required=True)
@click.option('-a', '--alias', 'alias', type=str, help='http endpoint alias name', required=True)
@click.option('-e', '--endpoint_id', 'endpoint_id', type=str, help='endpoint identifier', required=True)
@click.pass_context
def simple_update(ctx, model_id, version, image, deployment_id, alias, endpoint_id):
    versions = _create_version(ctx, model_id, version, image)
    click.echo("[info] create version and upload model. // version_id: " + versions['version_id'])

    service = _create_service(deployment_id, versions['version_id'], alias)
    service = _json_loads(service)
    click.echo("[info] create service. // service_id: " + service['service_id'])

    time.sleep(10.0)
    endpoint = _update_endpoint(deployment_id, service['service_id'], endpoint_id)
    click.echo("[info] update endpoint.")

    url = 'https://{}.api.dev.abeja.io/deployments/{}'.format(CONFIG['organization-name'], deployment_id)
    click.echo("[info] url: " + url)


# ---------------------------------------------------
# common
# ---------------------------------------------------
def _get(url):
    try:
        r = requests.get(url, headers={'Content-Type': 'application/json'},
                         auth=(ABEJA_PLATFORM_USER_ID, ABEJA_PLATFORM_TOKEN))
        r.raise_for_status()
        r = r.text

    except requests.exceptions.HTTPError as e:
        _error_message(e, r)
        r = None

        sys.exit(0)

    return r


def _get_data(url, json_data):
    try:
        r = requests.get(url, json_data, headers={'Content-Type': 'application/json'},
                         auth=(ABEJA_PLATFORM_USER_ID, ABEJA_PLATFORM_TOKEN))
        r.raise_for_status()
        r = r.text

    except requests.exceptions.HTTPError as e:
        _error_message(e, r)
        r = None

        sys.exit(0)

    return r


def _post(url, json_data):
    try:
        r = requests.post(url, json_data, headers={'Content-Type': 'application/json'},
                          auth=(ABEJA_PLATFORM_USER_ID, ABEJA_PLATFORM_TOKEN))
        r.raise_for_status()
        r = r.text

    except requests.exceptions.HTTPError as e:
        _error_message(e, r)
        r = None

        sys.exit(0)

    return r


def _delete(url):
    try:
        r = requests.delete(url, headers={'Content-Type': 'application/json'},
                            auth=(ABEJA_PLATFORM_USER_ID, ABEJA_PLATFORM_TOKEN))
        r.raise_for_status()
        r = r.text

    except requests.exceptions.HTTPError as e:
        _error_message(e, r)
        r = None

        sys.exit(0)

    return r


def _patch(url, jsonData):
    try:
        r = requests.patch(url, jsonData, headers={'Content-Type': 'application/json'},
                           auth=(CONFIG['abeja-platform-user'], CONFIG['personal-access-token']))
        r.raise_for_status()
        r = r.text

    except requests.exceptions.HTTPError as e:
        _error_message(e, r)
        r = None

        sys.exit(0)

    return r


def _error_message(e, r):
    click.echo(
        '{} {} {} {} {} {} {}'.format("[error]", date, "request.url =>", str(r.request.method), str(r.request.url),
                                      str(e.response.status_code), str(e.response.reason)))
    click.echo('{} {} {} {}'.format("[error]", date, "request.headers =>", str(r.request.headers)))
    click.echo('{} {} {} {}'.format("[error]", date, "request.body =>", str(r.request.body)))
    click.echo('{} {} {} {}'.format("[error]", date, "response.header =>", str(r.headers)))
    click.echo('{} {} {} {}'.format("[error]", date, "response.body =>", str(r.text)))


def _download(name, url):
    file_name = url.split("/")[-1]
    file_path = '{}/{}'.format(name, file_name)

    res = requests.get(url, stream=True)
    if res.status_code == 200:
        with open(file_path, 'wb') as file:
            res.raw.decode_content = True
            shutil.copyfileobj(res.raw, file)


def _json_loads(r):
    try:
        r = json.loads(r)
    except json.JSONDecodeError as e:
        click.echo('{} {} {} {}'.format("[error]", date, "error message =>", e))

    return r


def _json_file_load(path):
    with open(path, 'r') as f:
        json_data = json.load(f)
    return json_data


if __name__ == '__main__':
    main()
